# Copyright 2021 Florian Fischer
"""Coordniator to synchronize echoclients"""
import subprocess
import logging

from .globalvars import CLIENT_DIR

log = logging.getLogger(__name__)

COORDINATOR_EXE = f'{CLIENT_DIR}/build/apps/coordinator'


class Coordinator(subprocess.Popen):
    """Coordniator class usable in a with statement"""
    def __init__(self, nclients):
        cmd = f'{COORDINATOR_EXE} {nclients}'
        log.debug('Creating coordinator object with cmd: %s', cmd)
        super().__init__(cmd.split(),
                         stdout=subprocess.DEVNULL,
                         stderr=subprocess.DEVNULL)

    def __enter__(self):
        log.debug('Start coordinator')
        super().__enter__()

    def __exit__(self, exc_type, value, traceback):  # pylint: disable=arguments-differ
        log.debug('Join coordinator')
        super().__exit__(exc_type, value, traceback)
