# Copyright 2021 Florian Fischer
"""Utility functions"""

import logging
import os
import subprocess
from typing import Dict, MutableMapping


def prepare_env(update_env: MutableMapping) -> Dict:
    """Update and return the a copy of os.environ with a new mapping"""
    current_env = dict(os.environ)
    current_env.update(update_env)
    return current_env


def cmd_run(cmd: str, log: logging.Logger, cwd=None, env=None, check=True):
    """Run a verbosity aware subcommand"""
    stdout = None
    if log.getEffectiveLevel() > logging.DEBUG:
        stdout = subprocess.DEVNULL

    log.debug('Running subcommand: %s', cmd)

    try:
        subprocess.run(cmd.split(),
                       stdout=stdout,
                       stderr=subprocess.PIPE,
                       text=True,
                       env=env,
                       check=check,
                       cwd=cwd)

    except subprocess.CalledProcessError as run_err:
        log.critical('subcommand failed: %s', run_err)
        log.debug('%s', run_err.stderr)
        raise run_err
