#include <assert.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>

int main(int argc, char* argv[]) {
	size_t cons;
	if (argc != 4) {
		fprintf(stderr, "Usage: %s <addr> <port> <cons>\n", argv[0]);
		exit(EXIT_FAILURE); 
	}

	struct addrinfo* server;

	int err = getaddrinfo(argv[1], argv[2], NULL, &server);
	if (err) {
		if (err == EAI_SYSTEM) {                                                    
			perror("getaddrinfo failed");
		} else {                                                                    
			fprintf(stderr, "error in getaddrinfo: %s\n", gai_strerror(err));
		}
		exit(EXIT_FAILURE); 
	}

	cons = strtol(argv[3], NULL, 10);
	int* sockets = malloc(sizeof(int) * cons);
	if (!sockets) {
		perror("malloc failed");
		exit(EXIT_FAILURE);
	}

	printf("connect phase...\n");
	for (size_t i = 0; i < cons; ++i) {
		sockets[i] = socket(server->ai_family, server->ai_socktype, server->ai_protocol);
		if (sockets[i] < 0) {
			perror("socket failed");
			exit(EXIT_FAILURE);
		}

		err = connect(sockets[i], server->ai_addr, server->ai_addrlen);
		if (err) {
			perror("connect failed");
			exit(EXIT_FAILURE);
		}
	}

	printf("echo check...\n");

	// test connectivity
	for (size_t i = 0; i < cons; ++i) {
		char out[32];
		char in[32];
		assert(send(sockets[i], &out, 32, 0) == 32);
		assert(recv(sockets[i], &in, 32, 0) == 32);
		assert(memcmp(&out, &in, 32) == 0);
	}

	printf("close phase...\n");

	for (size_t i = 0; i < cons; ++i) {
		shutdown(sockets[i], SHUT_RDWR);
		close(sockets[i]);
	}

	free(sockets);

	return 0;
}
