package main

import (
	"flag"
	"log"
	"net"
	"os"
	"strconv"
	"time"
)

func main() {
	port := flag.Int("port", 12345, "Port to accept connections on.")
	host := flag.String("host", "0.0.0.0", "Host or IP to bind to")
	computation := flag.Uint("computation", 0, "Computation done before sending a responce in microseconds")
	flag.Parse()

	l, err := net.Listen("tcp", *host+":"+strconv.Itoa(*port))
	if err != nil {
		log.Panicln(err)
	}
	log.Println("Listening to connections at '"+*host+"' on port", strconv.Itoa(*port), "with", *computation, "microseconds computation")
	defer l.Close()

	for {
		conn, err := l.Accept()
		if err != nil {
			log.Panicln(err)
		}

		go handleRequest(conn, *computation)
	}
}

func handleRequest(conn net.Conn, computation uint) {
	defer conn.Close()

	for {
		buf := make([]byte, 1024)
		size, err := conn.Read(buf)
		if err != nil {
			return
		}

		if size >= 5 && string(buf[:5]) == "quit\n" {
			os.Exit(0)
		}

		deadline := time.Now().Add(time.Microsecond * time.Duration(computation))
		for time.Until(deadline) > 0 {
		}

		conn.Write(buf[:size])
	}
}
