use std::env;
use std::time::{Duration, Instant};

use tokio::net::TcpListener;
use tokio::io::{AsyncReadExt, AsyncWriteExt};

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let args: Vec<String> = env::args().collect();
    let computation = match args.len() {
        1 => 0,

        2 => args[1].parse()?,

        _=> {
            let prog = &args[0];
            panic!("Usage: {} [computation in us]", prog);
        }
    };

    let mut runtime_builder = tokio::runtime::Builder::new_multi_thread();

    let worker_key = "TOKIO_WORKER_COUNT";
    match env::var_os(worker_key) {
        Some(val) => {
            let worker_str = val.into_string().unwrap();
            runtime_builder.worker_threads(worker_str.parse()?);
        }
        None => {}
    };

    runtime_builder.enable_all().build().unwrap().block_on(async {
        let listener = TcpListener::bind("[::]:12345").await.unwrap();
        println!("Echoserver listening on :::12345 with {} microseconds computation", computation);

        loop {
            let (mut socket, _) = listener.accept().await.unwrap();

            tokio::spawn(async move {
                let mut buf = [0; 1024];

                // In a loop, read data from the socket and write the data back.
                loop {
                    let n = match socket.read(&mut buf).await {
                        // socket closed
                        Ok(n) if n == 0 => return,
                        Ok(n) => n,
                        Err(e) => {
                            eprintln!("failed to read from socket; err = {:?}", e);
                            return;
                        }
                    };

                    if n == 5 && &buf[0..n] == "quit\n".as_bytes() {
                        std::process::exit(0);
                    }

                    let now = Instant::now();
                    let execution_duartion = Duration::from_micros(computation);
                    while now.elapsed() < execution_duartion {}

                    // Write the data back
                    if let Err(e) = socket.write_all(&buf[0..n]).await {
                        eprintln!("failed to write to socket; err = {:?}", e);
                        return;
                    }
                }
            });
        }
   })
}
